/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package applet;

import java.awt.Button;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.TextComponent;
import java.awt.TextField;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import rules.RuleSet;

import matcher.HanziScanner;
import matcher.TibiListener;

public class TibiComponent extends Container implements TibiListener {

	static final int n=5;
	
	TextComponent text;
	ButtonBar buttons;
	Label status;
	WritingCanvas canvas;
	Button clearButton;

	public TibiComponent(RuleSet rules) {		
		
		int ruleCount= rules.visibleRuleCount();
		HanziScanner scanner= new HanziScanner(rules, n);
		
		GridBagLayout gridbag= new GridBagLayout();
		GridBagConstraints c= new GridBagConstraints();
		
	    setFont(new Font("Arial", Font.PLAIN, 20));
	    setLayout(gridbag);
	    
	    text= new TextField("",20);
	    c.fill = GridBagConstraints.HORIZONTAL;
	    c.gridy=0;
	    c.weighty=0.1;
	    gridbag.setConstraints(text, c);
	    add(text);
	    
	    canvas= new WritingCanvas(scanner);
	    buttons= new ButtonBar(text, canvas, n);
	    c.fill = GridBagConstraints.BOTH;
	    c.gridy=1;
	    c.weighty=1.0;
	    gridbag.setConstraints(buttons, c);
	    add(buttons);
	    buttons.setPreferredSize(new Dimension(300,50));
	    
	    status= new Label();
	    status.setFont(new Font("Arial", Font.PLAIN, 12));
	    c.fill = GridBagConstraints.BOTH;
	    c.gridy=2;
	    c.weighty=0.1;
	    gridbag.setConstraints(status, c);
	    add(status);	    
	    
	    c.fill = GridBagConstraints.BOTH;
	    c.gridy=3;
	    c.weighty=4.0;
	    gridbag.setConstraints(canvas, c);
	    canvas.setPreferredSize(new Dimension(300,250));
	    add(canvas);
	    
	    clearButton= new Button("Clear");
	    c.fill = GridBagConstraints.BOTH;
	    c.gridy=4;
	    c.weighty=1.0;
	    gridbag.setConstraints(clearButton, c);
	    clearButton.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				canvas.clear();
				buttons.clear();
			}	    	
	    });
	    clearButton.setPreferredSize(new Dimension(300,50));
	    add(clearButton);
	    status.setText(ruleCount + " rules loaded");

		scanner.addListener(this);
	}
	
	public void setResult(int n, String value) {
		buttons.setResult(n, value);
	}
	
	public void status (int type, String message) {
		switch(type) {
		case TibiListener.START:
			message="Searching...";
			break;
		case TibiListener.CLEAR:
		case TibiListener.FINISHED:
			message="";
			break;
		case TibiListener.STOP:
			message="stopped...";
			break;
		case TibiListener.ERROR:
			message="Error: "+message;
			break;
		}
		status.setText(message);
		status.repaint();
	}
}
