/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package applet;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import nodes.Vector2d;

import matcher.HanziScanner;

public class WritingCanvas extends Canvas {

	Vector<Vector<Vector2d>> path;
	HanziScanner scanner;
	Thread thread;
	
	class Mausi implements MouseListener, MouseMotionListener {

		WritingCanvas master;
		Vector<Point> stroke;
		Mausi (WritingCanvas who) {
			master= who;
		}
		public void mouseClicked(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		public void mousePressed(MouseEvent e) {
			stroke= new Vector<Point>();
			stroke.add(e.getPoint());
			scanner.stopNextOccasion();
		}
		public void mouseReleased(MouseEvent e) {
			master.addStroke(stroke);
			stroke=null;
		}
		public void mouseDragged(MouseEvent e) {
			if (stroke!=null) {
				Graphics gc= master.getGraphics();
				gc.setColor(Color.BLACK);
				gc.drawLine(
						stroke.lastElement().x, stroke.lastElement().y, 
						e.getPoint().x, e.getPoint().y);
				stroke.add(e.getPoint());
			}
		}
		public void mouseMoved(MouseEvent e) {}
	}
	
	public WritingCanvas(HanziScanner scanner) {
		setBackground(Color.WHITE);
		Mausi mausi= new Mausi(this);
		addMouseListener(mausi);
		addMouseMotionListener(mausi);
		this.scanner= scanner;
		clear();
	}

	public void clear() {
		try {
			if (thread!= null && thread.isAlive()) {
				scanner.stopNextOccasion();
				thread.interrupt();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		path= new Vector<Vector<Vector2d>>();
		repaint();
		scanner.clear();
	}
	
	void addStroke(Vector<Point> stroke) {
		if (stroke.size()<2)
			return;
		double scale= 1.0/ Math.max(getBounds().getHeight(), getBounds().getWidth());
		
		path.add(new Vector<Vector2d>());
		for (Point p : stroke) {
			Vector2d q= new Vector2d();
			q.x= scale*p.x;
			q.y= scale*p.y;
			path.lastElement().add(q);
		}
		
		try {
			if (thread!= null && thread.isAlive()) {
				scanner.stopNextOccasion();
				thread.join();
			}
			scanner.addStroke(path.lastElement());
			thread= new Thread(scanner);
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
		
	public void paint(Graphics gc) {
		gc.setColor(Color.BLACK);
		double scale= Math.max(getBounds().getHeight(), getBounds().getWidth());
		for (Vector<Vector2d> stroke : path) {
			Vector2d q= null;
			for (Vector2d p : stroke) {
				if (q!=null)
					gc.drawLine((int)(scale*q.x), (int)(scale*q.y), (int)(scale*p.x), (int)(scale*p.y));
				q=p;
			}
		}
	}
}
