/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package matcher;

import nodes.Vector2d;

public final class BoundingBox {
	public double xmin, xmax, ymin, ymax;
	
	public BoundingBox() {
		xmin=ymin=Double.POSITIVE_INFINITY;
		xmax=ymax=Double.NEGATIVE_INFINITY;
	}

	public BoundingBox(BoundingBox box) {
		xmin= box.xmin;
		xmax= box.xmax;
		ymin= box.ymin;
		ymax= box.ymax;
	}
	
	public BoundingBox(Vector2d p, Vector2d q) {
		this();
		expand(p);
		expand(q);
	}
	
	public void expand(Vector2d p) {
		expand(p.x, p.y);
	}
	
	public void expand(double px, double py) {
		if (px < xmin) xmin=px; 
		if (py < ymin) ymin=py; 
		if (px > xmax) xmax=px; 
		if (py > ymax) ymax=py;
	}
	
	public void expand (BoundingBox box) {
		expand(box.xmin, box.ymin);
		expand(box.xmax, box.ymax);
	}
	
	public boolean isActive() {
		return (xmax>=xmin) && (ymax>=ymin);
	}
	
	public String toString() {
		return "["+xmin+":"+xmax+", "+ymin+":"+ymax+"]";
	}
}
