/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package matcher;

import rules.Locator;

public final class ScaledLocator extends Locator{
	
	public ScaledLocator(Locator loc, BoundingBox box) {
		super(loc.getName(),
				box.xmin + 0.1*loc.getMinX()*(box.xmax-box.xmin),
				box.xmin + 0.1*loc.getMaxX()*(box.xmax-box.xmin),
				box.ymin + 0.1*loc.getMinY()*(box.ymax-box.ymin),
				box.ymin + 0.1*loc.getMaxY()*(box.ymax-box.ymin));
	}
	
	public double getFit(ScaledLocator loc) {
		double punish= ParameterSet.locatorRangeDecay;
		double fit=1.0;
		if (getMaxX() < loc.getMinX())
			fit = Math.min(fit, 1-punish*(loc.getMinX()-getMaxX()));
		if (getMaxY() < loc.getMinY())
			fit = Math.min(fit, 1-punish*(loc.getMinY()-getMaxY()));
		if (getMinX() > loc.getMaxX())
			fit = Math.min(fit, 1-punish*(getMinX()-loc.getMaxX()));
		if (getMinY() > loc.getMaxY())
			fit = Math.min(fit, 1-punish*(getMinY()-loc.getMaxY()));
		
		return Math.max(fit,0);
	}
	
	public static double shapeFit(Locator loc, BoundingBox box) {
		double shape= (box.xmax-box.xmin)/(box.xmax-box.xmin+box.ymax-box.ymin);
		//System.out.println("Shape: "+shape+"  ("+loc.getMinX()+"-"+loc.getMaxX()+")");
		double fit=Math.min(1- ParameterSet.locatorRangeDecay * (0.1*loc.getMinX()- shape),
						    1- ParameterSet.locatorRangeDecay * (shape - 0.1*loc.getMaxX()));
		fit = Math.max(fit, 0.0);
		fit = Math.min(fit, 1.0);
		//System.out.println("shape fit: "+fit);
		return fit;
	}
	
	public String toString() {
		return "*"+super.toString();	
	}
}
