/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package nodes;

import java.text.DecimalFormat;

import rules.StrokePattern;

public final class Node extends Vector2d {

	public int typeset;

	Node(Vector2d p, int type) {
		super(p);
		this.typeset=type;
	}
	
	Node(double x, double y, int type) {
		super(x, y);
		this.typeset= type;
	}
	
	public String toString() {
		StringBuffer buf= new StringBuffer();
		if (typeset==0)
			buf.append("T");
		else
			buf.append(StrokePattern.toString(typeset));
		DecimalFormat f= new DecimalFormat("#");
		buf.append(f.format(10*x)+":"+f.format(10*y));
		return buf.toString();
	}

	static double factor1 = Math.sin(3.0/16.0 * Math.PI);
	static double factor2 = Math.sin(3.5/16.0 * Math.PI);
	
	/** Returns a BitSet of considerable directions.
	 * 
	 */
	public static int getStrokeType(Vector2d vec, int prefdir) {
		int dir=0;
		for (int i=1; (i&255)!=0; i<<=1) {
			double factor= ((i & prefdir)==0) ? factor1 : factor2;
			if (vec.x>0 && Math.abs(vec.y)<= factor * vec.x) {
				dir = dir | i;
			}
			double x2= vec.x - vec.y;
			double y2= vec.x + vec.y;
			vec.x=x2;
			vec.y=y2;					
		}
		return dir;
	}

	
	static final long serialVersionUID = -8692178433128243738L;
}
