/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package nodes;

import java.util.BitSet;
import java.util.Vector;


public final class NodeListTraverser {
	
	Vector<Node[]> nodes;
	BitSet visited;
	int pos;
	boolean variations;
	boolean inverted;
	
	public NodeListTraverser(NodeList nodeList) {
		nodes= nodeList.nodes;
		visited= new BitSet(nodes.size());
		pos=0;
		variations= true;
		inverted= false;
	}
	
	public NodeListTraverser(NodeListTraverser view) {
		nodes= view.nodes;
		visited= new BitSet(view.visited.size());
		visited.or(view.visited);
		pos= view.pos;
		variations= view.variations;
		if (view.inverted) 
			System.err.println("Inverted NodeTraverser was spawned.");
	}
	
	public NodeListTraverser copy() {
		return new NodeListTraverser(this);
	}
	
	public boolean hasVariations() {
		return variations;
	}
	
	public void variationsChecked() {
		variations=false;
		inverted=false;
	}
	
	public void invert() {
		inverted= true;
	}
	
	public NodeTraverser next(int pref) {
		pos= pref;
		visited.set(pos);
		NodeTraverser result= new NodeTraverser(nodes.elementAt(pos), inverted);
		variations= true;
		inverted= false;
		return result;
	}
	
	public void skip() {
		if (nodes.size() > visited.size())
			visited.clear(visited.size(), nodes.size());
		if (visited.get(pos)) {
			pos=visited.nextClearBit(0);
			pos=visited.nextClearBit(pos+1);
		} else {
			pos= visited.nextClearBit(pos+1);			
		}
		inverted= false;
		variations= true;
	}
	
	public NodeTraverser next() {
		if (nodes.size() > visited.size())
			visited.clear(visited.size(), nodes.size());
		if (visited.get(pos))
			return next(visited.nextClearBit(0));
		else
			return next(pos);
	}
	
	public boolean isInverted() {
		return inverted;
	}
	
	public boolean isActive() {
		return !isComplete() && (pos < nodes.size());
	}
	
	public boolean isComplete() {
		return visited.cardinality() == nodes.size();
	}

	public String toString() {
		return (inverted ? "-" : "") + pos + "/" +visited;
	}	
}
