/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package nodes;

public final class NodeTraverser {

	Node[] nodes;
	boolean inverted, matched;
	int pos, topos, blocked;
	
	public NodeTraverser(Node[] nodes, boolean inverted) {
		this.nodes= nodes;
		this.inverted= inverted;
		clear();
	}
	
	public NodeTraverser(NodeTraverser traverser) {
		nodes= traverser.nodes;
		inverted= traverser.inverted;
		matched= traverser.matched;
		pos= traverser.pos;
		topos= traverser.topos;
		blocked= traverser.blocked;
	}
	
	public NodeTraverser copy() {
		return new NodeTraverser(this);
	}
	
	/** return to strat */
	public void clear() {
		if (inverted)
			pos= nodes.length;
		else
			pos=-1;
		topos=-1;
		matched=false;		
	}
	
	public Node first() {
		if (inverted)
			return nodes[nodes.length-1];
		else
			return nodes[0];
	}
	
	public Node last() {
		if (inverted)
			return nodes[0];
		else
			return nodes[nodes.length-1];
	}
	
	public Vector2d strokeEnd() {
		return nodes[pos+ (inverted?-1:+1)];
	}

	public Vector2d strokeEnd(int dir) {
		int oldpos= pos;
		int lastmatch= pos;
		while (next()) {
			if (!matchesStrokeType(dir)) {
				break;
			} else {
				lastmatch= pos;
			}
		}
		matched=true;
		blocked=dir;
		topos=lastmatch + (inverted?-1:+1);
		pos= oldpos;
		return nodes[topos];
	}
	
	public Vector2d strokeStart() {
		return nodes[pos];
	}
	
	public boolean matchesStrokeType(int type) {
		if (matched && (type&blocked)!=0)
			return false;
		int typeset;
		if (inverted)
			typeset=nodes[pos-1].typeset;
		else
			typeset=nodes[pos].typeset;
			
		if (inverted)
			type= ((type << 4) | (type >> 4)) & 255;
		
		return (typeset & type) != 0;
	}

	public boolean isMatched() {
		return matched;
	}
	
	public boolean next() {
		if (inverted) { 
			pos--;
			if (topos==pos) matched=false;
			return pos> 0;
		} else {
			pos++;
			if (topos==pos) matched=false;
			return pos< nodes.length-1;
		}
	}
	
	public boolean isInverted() {
		return inverted;
	}
}
