package parser;
   
import java_cup.runtime.*;
      
%%

%class Lexer
%line
%column
%unicode    
%cup
   
%{   
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }
    private Symbol symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }
%}
   

EOL = \r | \n | \r\n
SPACE = {EOL} | [ \t\f]
Comma = "," | \uff0c
Colon = ":" | \uff1a
Semi = ";" | \uff1b
LPar = "(" | \uff08
RPar= ")" | \uff09
LBra= "[" | \uff3b
RBra= "]" | \uff3d
LCurl= "{" | \uff5b
RCurl= "}" | \uff5d
NUMBER = [+-]? ([0-9]+\.? | [0-9]*\.[0-9]+)
NAME = [A-Za-z_]*
LITERAL = {NAME} | [\u0080-\ufffd]+
QUOTED = \"[^\"]+\" | \'[^\']+\'
Comment = %[^\n]*
   
%%
/* ------------------------Lexical Rules Section---------------------- */

<YYINITIAL> {
	{Colon}			{ return symbol(sym.COLON); }
	{Semi}			{ return symbol(sym.SEMI); }
	{Comma}			{ return symbol(sym.COMMA); }
	"E"				{ return symbol(sym.E); }
	"NE"			{ return symbol(sym.NE); }
	"N"				{ return symbol(sym.N); }
	"NW"			{ return symbol(sym.NW); }
	"W"				{ return symbol(sym.W); }
	"SW"			{ return symbol(sym.SW); }
	"S"				{ return symbol(sym.S); }
	"SE"			{ return symbol(sym.SE); }
	"\*"			{ return symbol(sym.STAR); }
	"\|"			{ return symbol(sym.BAR); }
	"#"				{ return symbol(sym.HASH); }
	"-"				{ return symbol(sym.MINUS); }
	{RPar}			{ return symbol(sym.RPAREN); }
	{LPar}			{ return symbol(sym.LPAREN); }
	{RBra}			{ return symbol(sym.RBRACK); }
	{LBra}			{ return symbol(sym.LBRACK); }
	{LCurl}			{ return symbol(sym.LCURL); }
	{RCurl}			{ return symbol(sym.RCURL); }
	{NUMBER}		{ return symbol(sym.NUMBER, new Double(yytext())); }
    {LITERAL} 		{ return symbol(sym.LITERAL, yytext()); }
    {QUOTED}		{ return symbol(sym.LITERAL, yytext().substring(1,yytext().length()-1)); }
    {SPACE}  		{}
    {Comment}  		{}   
}


/* No token was found for the input so through an error.  Print out an
   Illegal character message with the illegal character that was found. */
[^]                    { throw new Error("Illegal character <"+yytext()+">"); }
