/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package rules;

import java.util.HashMap;
import java.util.Map;

public class Locator {
	String name;

	static Map<String,String> nameMap= new HashMap<String,String>();
	double xmin,ymin,xmax,ymax;
	
	public Locator(String name) {
		// keep only one instance per name, thus efficient == operator can be used to compare names 
		String name2= nameMap.get(name);
		if (name2==null) {
			this.name=name;
			nameMap.put(name,name);
		} else {
			this.name=name2;
		}
	}
	
	public Locator(String name, double xmin, double xmax, double ymin, double ymax) {
		this(name);
		setXRange(xmin, xmax);
		setYRange(ymin, ymax);
	}
	
	public void setXRange(double min, double max){
		xmin=min;
		xmax=max;
	}
	
	public void setYRange(double min, double max){
		ymin=min;
		ymax=max;
	}
	
	public String getName() {
		return name;
	}
	
	public double getMinX() {
		return xmin;
	}
	
	public double getMinY() {
		return ymin;
	}
	
	public double getMaxX() {
		return xmax;
	}
	
	public double getMaxY() {
		return ymax;
	}
	
	public static Locator fullRangeLocator(String name) {
		return new Locator(name, 0, 10, 0, 10);
	}
	
	public String toString() {
		StringBuffer buf=new StringBuffer("["+name+" ");
		if (xmin != 0.0)
			buf.append(xmin);
		buf.append(":");
		if (xmax!= 10.0)
			buf.append(xmax);
		buf.append(", ");
		if (ymin != 0.0)
			buf.append(ymin);
		buf.append(":");
		if (ymax != 10.0)
			buf.append(ymax);
		buf.append("]");
		return buf.toString();
	}
}
