package rules;

import java.util.Vector;

import util.BiStack;


public class PatternTraverser {
	
	BiStack<Vector<Pattern>> args;
	BiStack<BiStack<Pattern>> stack;
	Pattern current;
	
	public PatternTraverser(Pattern pattern) {
		args= new BiStack<Vector<Pattern>>();
		stack= new BiStack<BiStack<Pattern>>();
		stack.addFirst(new BiStack<Pattern>());
		stack.peek().addFirst(pattern);
		current=null;
	}

	public PatternTraverser(PatternTraverser traverser) {
		stack= traverser.stack.copy();
		args= traverser.args.copy();
		current=traverser.current;		
	}
	
	public Pattern next() {
		if (! isLastInGroup()) {
			current= stack.peek().peek();
			stack.addFirst(new BiStack<Pattern>());
			if (current instanceof GroupPattern) {
				stack.peek().set((GroupPattern)current);
			} else if (current instanceof ReferencePattern) {
				stack.peek().addFirst(((ReferencePattern)current).getPattern());
				args.addFirst(new Vector<Pattern>());
				if (current instanceof MacroPattern) {
					for (Pattern p : ((MacroPattern) current))
						args.peek().add(p);
				}
			} else if (current instanceof ArgumentPattern) {
				stack.peek().addFirst(args.peek().elementAt(((ArgumentPattern)current).getArgNumber()-1));
				args.hide();
			}
		} else {
			stack.removeFirst();
			stack.addFirst(stack.poll().copy());
			current= stack.peek().poll();
			if (current instanceof ReferencePattern)
				args.removeFirst();
			if (current instanceof ArgumentPattern)
				args.unhide();
		}
		return current;
	}

	public void skipChildren() {
		stack.removeFirst();
		stack.addFirst(new BiStack<Pattern>());
	}
	
	public boolean isLastInGroup() {
		return stack.peek().peek() == null;
	}
	
	public Pattern current() {
		return current;
	}
	
	public boolean hasNext() {
		return (stack.size()>1 || stack.peek().size()>0);
	}	
}