/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package rules;

public class StrokePattern extends Pattern {

	public static int E=0;
	public static int NE=1;
	public static int N=2;
	public static int NW=3;
	public static int W=4;
	public static int SW=5;
	public static int S=6;
	public static int SE=7;
	
	static String directions[]={"E","NE","N","NW","W","SW","S","SE"};
	
	int stroke;
	boolean connected;
	
	public StrokePattern(int strokeType) {
		stroke= 1<<strokeType;
		connected= false;
	}

	public StrokePattern(int strokeType, boolean connect) {
		stroke= 1<<strokeType;
		connected= connect;
	}
	
	public void matchAlso(int orientation) {
		stroke= stroke | (1<<orientation);
	}
	
	public int getStrokeBits() {
		return stroke;
	}
	
	public boolean matches(int strokeBits) {
		return (strokeBits & stroke) != 0;
	}
	
	public boolean isConnected() {
		return connected;
	}
	
	public void setConnected(boolean connect) {
		connected= connect;
	}
	
	public static String toString(int stroke) {
		if (stroke==0)
			return "T";
		String result="";
		for (int i=0; i<8; ++i) {
			if ((stroke & (1<<i)) != 0) {
				if (result.length()>0)
					result= result + "|";
				result= result + directions[i];
			}
		}
		return result;
	}

	public String toString() {
		if (connected)
			return "-" + toString(stroke);
		else
			return toString(stroke);
	}
}
