/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package test;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Timer;
import java.util.Vector;

import nodes.Vector2d;

import matcher.HanziScanner;
import matcher.ParameterSet;
import matcher.TibiListener;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import parser.RuleSetReader;
import rules.RuleSet;


public class FailBrowser {

	static final String testfile="testcases.txt";
	static final DecimalFormat f= new DecimalFormat("#.####");
	Shell shell,parent;
	Canvas canvas;

	class TestCaseListener implements MouseListener  {
		Vector<Vector<Vector2d>> path;
		public TestCaseListener(Vector<Vector<Vector2d>> path) { this.path=path; }
		public void mouseUp(MouseEvent e) {}
		public void mouseDown(MouseEvent e) { 
			TibiTestcaseManager.canvas.setPath(path);;
			TibiTestcaseManager.canvas.scan();
			TibiTestcaseManager.canvas.redraw();
			TibiTestcaseManager.canvas.update();
		}
		public void mouseDoubleClick(MouseEvent e) {}										
	}
	
	class ResultListener implements TibiListener {
		String result[];
		
		ResultListener() {
			clearAllResults();
		}
		public void clearAllResults() {
			result= new String[1];
		}
		public void setResult(int n, String result) {
			this.result[n]=result;
		}
		public void status(int type, String message) {
			if (type==TibiListener.ERROR)
				System.out.println("Error: "+message);
		}
		String get(int i) {
			return result[i];
		}
	}
	
	FailBrowser(Shell parent) {
		this.parent=parent;
	}

	void open() {
		shell.open();
	}
	
	int runtest() {
		ParameterSet.verbose=false;
		if (shell==null || shell.isDisposed()) {
			shell= new Shell(parent, SWT.RESIZE | SWT.BORDER | SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX );
			shell.setLayout(new FillLayout());
			shell.setSize(250,300);
			canvas= new Canvas(shell, SWT.V_SCROLL);
			RowLayout layout= new RowLayout(SWT.HORIZONTAL);
			layout.pack=true;
			layout.justify= true;
			layout.fill=true;
			canvas.setLayout(layout);
		} else {
			for (Control child : canvas.getChildren())
				child.dispose();
		}
		
		long start= System.currentTimeMillis();
		int count=0;
		try {
			BufferedReader in = new BufferedReader(
					new	InputStreamReader(
							new FileInputStream(testfile), "UTF-8"));
			HanziScanner scanner= new HanziScanner(RuleSetReader.readFile("pattern.txt"), 1);
			ResultListener result= new ResultListener();
			scanner.addListener(result);
			
			String literal;
			String line;
			int no=0;
			do {
				line= in.readLine();
				if (line!=null && line.trim().length()>0) {
					literal=line;
					
					Vector<Vector<Vector2d>> path= scanPath(in);
					scanner.clear();
					scanner.addStrokeList(path);
					
					scanner.run();
					
					String detected=result.get(0);
					if (detected!=null && !detected.equals(literal)) {
						Button button= new Button(canvas, SWT.FLAT);
						button.setText(literal);
						button.addMouseListener(new TestCaseListener(path));
						count++;
						System.out.println("Test: "+(no++)+": failed");
					} else {
						System.out.println("Test: "+(no++)+": success");
					}
				}
			} while (line != null && count<100);
			in.close();
	} catch (Exception x) {
		System.err.println("Error reading the test case file");
		x.printStackTrace();
	}
	long end=System.currentTimeMillis();
	System.out.println((end-start)/1000.0+" seconds");
	System.out.println(count + " Errors.");
	ParameterSet.verbose=true;
	return count;
}

	static Vector<Vector<Vector2d>> scanPath(BufferedReader in) throws IOException {
		Vector<Vector<Vector2d>> path= new Vector<Vector<Vector2d>>();
		String line= in.readLine();
		do{
			Vector<Vector2d> stroke= new Vector<Vector2d>();
			String coords[]= line.split(" ");
			for (int i=0; i<coords.length-1; i+=2) {
				stroke.add(new Vector2d(Double.valueOf(coords[i]),Double.valueOf(coords[i+1])));
			}
			Vector<Vector2d> orig= new Vector<Vector2d>(stroke);
			Vector2d alg= new Vector2d();
			for (int i=1; i<stroke.size()-1; ++i){
				alg.set(orig.elementAt(i));
				alg.scale(2.0);
				alg.add(orig.elementAt(i-1));
				alg.add(orig.elementAt(i+1));
				alg.scale(0.25);
				stroke.elementAt(i).set(alg);
			}
			path.add(stroke);
			line= in.readLine();
		} while (line != null && line.trim().length()!=0);
		return path;
	}
	
	static void store(String literal, Vector<Vector<Vector2d>> path) throws IOException {
		FileOutputStream outstream= new FileOutputStream(testfile, true);
		PrintWriter out= new PrintWriter(new OutputStreamWriter(outstream,"UTF-8"));
		out.println(literal);
		for (Vector<Vector2d> v : path) {
			for (Vector2d p : v) {
				out.print(f.format(p.x)+" "+f.format(p.y)+" ");
			}
			out.println();
		}
		out.println();
		out.close();
	}
}

