/* Tibi - Chinese handwriting recognition
 * Copyright (C) 2007 Stefan Dirnstorfer <stefan@dadim.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package test;

import java.util.Collection;
import java.util.Vector;

import nodes.Vector2d;

import matcher.HanziScanner;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import rules.RuleSet;



public class WritingCanvas extends Canvas {
	
	Thread thread;
	HanziScanner scanner;
	Vector<Vector<Vector2d>> path;
	GC gc;
	Point last;
	
	public WritingCanvas(Composite parent) {
		super(parent, SWT.NONE);
		
		setBackground(new Color(Display.getCurrent(),255,255,255));
		setSize(300,300);
		
		path= new Vector<Vector<Vector2d>>();
		gc = new GC(this);
		last=null;
		
		addMouseListener(new MouseListener(){
			public void mouseUp(MouseEvent e) {
				last= null;
				addStroke(path.lastElement());
				scan();
				redraw();
			}
			public void mouseDown(MouseEvent e) { 
				scanner.stopNextOccasion();
				path.add(new Vector<Vector2d>());
				double size= Math.min(gc.getClipping().width, gc.getClipping().height);
				path.lastElement().add(
						new Vector2d(e.x /size, e.y / size));
				last = new Point(e.x, e.y);
			}
			public void mouseDoubleClick(MouseEvent e) {}	
		});
		
		addMouseMoveListener(new MouseMoveListener(){
			public void mouseMove(MouseEvent e) { 
				if (last!=null && path!=null && path.size()>0) {
					double size= Math.min(gc.getClipping().width, gc.getClipping().height);
					path.lastElement().add(
							new Vector2d(e.x /size, e.y / size));
					gc.drawLine(last.x, last.y, e.x, e.y);
					last= new Point(e.x, e.y);
				}
			}
		});
		
		addPaintListener(new PaintListener(){
	        public void paintControl(PaintEvent e){
				double size= Math.min(gc.getClipping().width, gc.getClipping().height);
	        	for (Vector<Vector2d> v: path) {
	        		int lastx=-1;
	        		int lasty=-1;
	        		for (Vector2d p : v) {
	        			int x= (int) (p.x * size);
	        			int y= (int) (p.y * size);
	        			if (lastx >0 && lasty >0)
	        				e.gc.drawLine(lastx, lasty, x, y);
	        			lastx= (int) x;
	        			lasty= (int) y;
	        		}
	        	}
	        }
	    }); 
	}
	
	public void clear() {
		scanner.clear();
		path.clear();
		redraw();
	}
		
	public void scan() {
		try {
			if (thread!= null && thread.isAlive()) {
				scanner.stopNextOccasion();
				thread.join();
			}
			thread= new Thread(scanner);
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void addStroke(Vector<Vector2d> stroke) {
		scanner.addStroke(stroke);
	}
	
	public void setPath(Vector<Vector<Vector2d>> path) {
		clear();
		path= new Vector<Vector<Vector2d>>(path);
		scanner.addStrokeList(path);
		this.path= path;
		redraw();
	}

	public void setRules(RuleSet rules) {
		scanner= new HanziScanner(rules, 6);
		scanner.addListener(TibiTestcaseManager.charbar);
	}
	
	public void dispose() {
		gc.dispose();
		super.dispose();
	}
}
